@@ -1,5 +1,6 @@
 #!/usr/bin/python
 
+from typing import Awaitable
 from selve.commandClasses.command import CommeoCommandDevice, CommeoCommandGroup, CommeoCommandGroupMan
 from selve.commandClasses.common import CommeoParamGetEvent, CommeoParamSetEvent, CommeoServiceFactoryReset, CommeoServiceGetState, CommeoServiceGetVersion, CommeoServicePing, CommeoServiceReset
 from selve.commandClasses.sensor import CommeoSensorGetIDs, SensorDevice
@@ -123,14 +124,15 @@ class Gateway():
 
     async def executeCommand(self, command):
         a = CommeoServiceGetState()
-        self._executeCommand(a)
+        await self._executeCommand(a)
         _LOGGER.debug("Service State: " + str(a.status))
         if a.status == ServiceState.READY:
-            self._executeCommand(command)
+            return self._executeCommand(command)
         else:
             _LOGGER.error("Gateway not ready to execute command")
+        return None
 
-    def _executeCommand(self, command):
+    async def _executeCommand(self, command):
         """[summary]
         Execute the given command using the serial port.
         It opens a communication to the serial port each time a
@@ -176,7 +178,7 @@ class Gateway():
             self.ser.close()
             return None
 
-    def discover(self):
+    async def discover(self):
         """[summary]
             Discover all devices registered on the usb-commeo        
         """
@@ -190,37 +192,37 @@ class Gateway():
         retry_n = 0
         retry_m = 0
         while not hasattr(commandIveo, "ids") and retry_n <=num_retries:
-            commandIveo.execute(self)
+            await commandIveo.execute(self)
             retry_n += 1
             time.sleep(1)
         retry_n = 0
         retry_m = 0
         while not hasattr(commandCommeoActors, "ids") and retry_m <=num_retries:
-            commandCommeoActors.execute(self)
+            await commandCommeoActors.execute(self)
             retry_m += 1
             time.sleep(1)
         retry_n = 0
         retry_m = 0
         while not hasattr(commandCommeoGroups, "ids") and retry_m <=num_retries:
-            commandCommeoGroups.execute(self)
+            await commandCommeoGroups.execute(self)
             retry_m += 1
             time.sleep(1)
         retry_n = 0
         retry_m = 0
         while not hasattr(commandCommeoSenders, "ids") and retry_m <=num_retries:
-            commandCommeoSenders.execute(self)
+            await commandCommeoSenders.execute(self)
             retry_m += 1
             time.sleep(1)
         retry_n = 0
         retry_m = 0
         while not hasattr(commandCommeoSenSims, "ids") and retry_m <=num_retries:
-            commandCommeoSenSims.execute(self)
+            await commandCommeoSenSims.execute(self)
             retry_m += 1
             time.sleep(1)
         retry_n = 0
         retry_m = 0
         while not hasattr(commandCommeoSensors, "ids") and retry_m <=num_retries:
-            commandCommeoSensors.execute(self)
+            await commandCommeoSensors.execute(self)
             retry_m += 1
             time.sleep(1)
 
@@ -320,14 +322,15 @@ class Gateway():
 
     # GATEWAY STATE....
 
-    def gatewayState(self):
+    async def gatewayState(self):
         command = CommeoServiceGetState()
-        command.execute(self)
-        return command.status.name
+        await command.execute(self)
+        if hasattr(command, "status"):
+            return command.status.name
 
-    def getVersionG(self):
+    async def getVersionG(self):
         command = CommeoServiceGetVersion()
-        command.execute(self)
+        await command.execute(self)
         return command
 
     def getGatewayFirmwareVersion(self):
@@ -339,17 +342,17 @@ class Gateway():
     def getGatewaySpec(self):
         return self.getVersionG.spec
 
-    def resetGateway(self):
+    async def resetGateway(self):
         command = CommeoServiceReset()
-        command.execute(self)
+        await command.execute(self)
         while not command.executed:
             time.sleep(1)
         command = CommeoServiceGetState()
-        command.execute(self)
+        await command.execute(self)
         retries = 0
         while command.status != ServiceState.READY:
             retries = retries+1
-            command.execute(self)
+            await command.execute(self)
             if retries == 5:
                 break
             time.sleep(3)
@@ -358,39 +361,39 @@ class Gateway():
         else:
             _LOGGER.info("Gateway reset")
 
-    def factoryResetGateway(self):
+    async def factoryResetGateway(self):
         command = CommeoServiceFactoryReset()
-        command.execute(self)
+        await command.execute(self)
         if command.executed:
             _LOGGER.info("Factory reset successful")
         else:
             _LOGGER.info("Factory reset failed")
 
-    def setEvents(self, eventDevice, eventSensor, eventSender, eventLogging, eventDuty):
+    async def setEvents(self, eventDevice, eventSensor, eventSender, eventLogging, eventDuty):
         command = CommeoParamSetEvent(eventDevice, eventSensor, eventSender, eventLogging, eventDuty)
         command.execute(self)
 
-    def getEvents(self):
+    async def getEvents(self):
         command = CommeoParamGetEvent()
-        command.execute(self)
+        await command.execute(self)
         _LOGGER.debug("Events: " + str(command.eventDevice) + " " + str(command.eventSensor) + " " + str(command.eventSender) + " " + str(command.eventLogging) + " " + str(command.eventDuty))
 
 ## Actor
 
-    def sendCommandToActor(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+    async def sendCommandToActor(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
         command = CommeoCommandDevice(id, command, type, parameter)
-        command.execute(self)
+        await command.execute(self)
         return command.executed
 
-    def scanActorDevices(self):
+    async def scanActorDevices(self):
         commandStart = CommeoDeviceScanStart()
         commandStop = CommeoDeviceScanStop()
         commandResult = CommeoDeviceScanResult()
 
-        commandStart.execute(self)
-        commandResult.execute(self)
+        await commandStart.execute(self)
+        await commandResult.execute(self)
         while commandResult.scanState == ScanState.RUN or commandResult.scanState == ScanState.VERIFY:
-            commandResult.execute(self)
+            await commandResult.execute(self)
             time.sleep(1)
         if commandResult.scanState == ScanState.END_SUCCESS:
             if commandResult.noNewDevices > 0:
@@ -398,7 +401,7 @@ class Gateway():
 
         return {}
 
-    def saveActorDevices(self, ids):
+    async def saveActorDevices(self, ids):
         if len(ids) > 0:
             for id in ids:
                 commandSave = CommeoDeviceSave(id)
@@ -407,19 +410,19 @@ class Gateway():
                 self.addDevice(id, dev)
                 dev.saveDevice()
 
-    def deleteActorDevice(self, id):
+    async def deleteActorDevice(self, id):
         if self.is_id_registered(id):
             dev:ActorDevice = self.devices[id]
             dev.deleteDevice()
 
 ## Group
 
-    def sendCommandToGroup(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+    async def sendCommandToGroup(self, id, command, type=DeviceCommandTypes.MANUAL, parameter=0):
         command = CommeoCommandGroup(id, command, type, parameter)
-        command.execute(self)
+        await command.execute(self)
         return command.executed
 
-    def sendCommandToGroupMan(self, idMask, command, type=DeviceCommandTypes.MANUAL, parameter=0):
+    async def sendCommandToGroupMan(self, idMask, command, type=DeviceCommandTypes.MANUAL, parameter=0):
         command = CommeoCommandGroupMan(command, type, idMask, parameter)
-        command.execute(self)
+        await command.execute(self)
         return command.ids
\ No newline at end of file
